﻿Imports System.Drawing
Imports Un4seen.Bass
Imports Un4seen.Bass.Misc
Imports Un4seen.Bass.Misc.WaveForm

Namespace Service

    Public Class BassLibService

        Public Event WaveFormRendered As EventHandler(Of BitmapSource)
        Private StreamHandle As Integer = 0
        Private Wave As WaveForm
        Private WaveFormTargetSize As New Size(1024, 768)

        Public Function Init() As Boolean
            Return Bass.BASS_Init(-1, 44100, BASSInit.BASS_DEVICE_CPSPEAKERS, IntPtr.Zero)
        End Function
        Public Sub StopFile()
            Bass.BASS_ChannelStop(StreamHandle)
            Bass.BASS_StreamFree(StreamHandle)
        End Sub

        Public Function PlayFile(filename As String) As Boolean
            StreamHandle = Bass.BASS_StreamCreateFile(filename, 0, 0, BASSFlag.BASS_SAMPLE_FLOAT)
            If StreamHandle = 0 Then
                Return False
            End If
            Return Bass.BASS_ChannelPlay(StreamHandle, False)
        End Function

        Public Sub CreateWaveForm(filename As String, width As Integer, height As Integer)
            Dim proc As New WAVEFORMPROC(AddressOf WaveFormCallback)
            Wave = New WaveForm(filename, proc, Nothing)
            Wave.FrameResolution = 0.1F
            Wave.CallbackFrequency = 1000
            Wave.DrawWaveForm = WaveForm.WAVEFORMDRAWTYPE.Stereo
            Wave.RenderStart(True, BASSFlag.BASS_DEFAULT)
        End Sub

        Public Sub Shutdown()
            StopFile()
            Bass.BASS_Free()
        End Sub

        Private Sub WaveFormCallback(ByVal framesDone As Integer, ByVal framesTotal As Integer, ByVal elapsedTime As TimeSpan, ByVal finished As Boolean)
            If Wave IsNot Nothing Then
                ServiceManager.GetService(Of IMainWindowService)().GetDispatcher().Invoke(Sub() UpdateImage(finished))
            End If
        End Sub

        Private Sub UpdateImage(ByVal finished As Boolean)
            Dim bmps As BitmapSource = BitmapSourceFromImage(Wave.CreateBitmap(WaveFormTargetSize.Width, WaveFormTargetSize.Height, -1, -1, finished))
            RaiseEvent WaveFormRendered(Me, bmps)
        End Sub

        Public Function BitmapSourceFromImage(ByVal bitmap As Bitmap) As BitmapSource
            Dim bitmapData = bitmap.LockBits(New Rectangle(0, 0, bitmap.Width, bitmap.Height), Imaging.ImageLockMode.[ReadOnly], bitmap.PixelFormat)
            Dim bmps = BitmapSource.Create(bitmapData.Width, bitmapData.Height, bitmap.HorizontalResolution, bitmap.VerticalResolution, PixelFormats.Bgr24, Nothing, bitmapData.Scan0, bitmapData.Stride * bitmapData.Height, bitmapData.Stride)
            bitmap.UnlockBits(bitmapData)
            Return bmps
        End Function



    End Class

End Namespace